@@ -39,11 +39,12 @@ import pylibmc
 import inspect
 import threading
 import logging
+import subprocess
 import multiprocessing.connection
 import concurrent.futures
 from contextlib import contextmanager
 
-VERSION = "0.0.7"
+VERSION = "0.0.9"
 
 class StarryError(Exception):
     def __init__(self, output):
@@ -164,19 +165,31 @@ class LibMemcachedClient(pylibmc.ThreadMappedPool):
 class StarryServer(object):
     def __init__(self, database, address, cache=64, log=None):
         self.database = os.path.abspath(database)
+        self.address = address
+        self.cache = None
+        self.thread_cond = threading.Condition()
+        self.thread_lock = threading.RLock()
+        self.write_first = True
+        self.queue = []
+        self.logger = StarryLog(log) if log is not None else None
 
+    def __del__(self):
+        if self.cache is not None:
+            try:
+                subprocess.check_output(f"fuser -k {os.path.join(self.database, 'cache.sock')}", shell=True, stderr=subprocess.STDOUT, universal_newlines=True)
+            except:
+                pass
+
+    def start(self, max_workers=512, backlog=512, authkey=b'starry'):
         if not os.path.isdir(self.database):
             os.mkdir(self.database)
 
         if type(address) is str:
             _removeFile(address)
 
-        self.address = address
-        self.cache = None
-
         if int(cache) > 0:
             try:
-                os.system("kill -9 $(ps -ef | grep " + os.path.join(self.database, 'cache.sock') + " | awk '$0 !~/grep/ {print $2}') 2>/dev/null")
+                subprocess.check_output(f"fuser -k {os.path.join(self.database, 'cache.sock')}", shell=True, stderr=subprocess.STDOUT, universal_newlines=True)
             except:
                 pass
 
@@ -188,17 +201,12 @@ class StarryServer(object):
                 self.cache = LibMemcachedClient(os.path.join(self.database, "cache.sock"))
             except:
                 try:
-                    os.system("kill -9 $(ps -ef | grep " + os.path.join(self.database, 'cache.sock') + " | awk '$0 !~/grep/ {print $2}') 2>/dev/null")
+                    subprocess.check_output(f"fuser -k {os.path.join(self.database, 'cache.sock')}", shell=True, stderr=subprocess.STDOUT, universal_newlines=True)
                 except:
                     pass
 
                 self.cache = None
 
-        self.logger = StarryLog(log) if log is not None else None
-        self.thread_cond = threading.Condition()
-        self.thread_lock = threading.RLock()
-        self.queue = []
-
         backup_dir = os.path.join(self.database, ".backup/")
 
         if not os.path.isdir(backup_dir):
@@ -206,14 +214,6 @@ class StarryServer(object):
             os.mknod(os.path.join(backup_dir, "~.lock"))
             self.backup()
 
-    def __del__(self):
-        if self.cache is not None:
-            try:
-                os.system("kill -9 $(ps -ef | grep " + os.path.join(self.database, 'cache.sock') + " | awk '$0 !~/grep/ {print $2}') 2>/dev/null")
-            except:
-                pass
-
-    def start(self, max_workers=512, backlog=512, authkey=b'starry'):
         with multiprocessing.connection.Listener(self.address, backlog=backlog, authkey=authkey) as listener:
             with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
                 while True:
@@ -635,7 +635,7 @@ class StarryServer(object):
             return None
 
         try:
-            value = msgpack.loads(value, raw=False)
+            value = msgpack.unpackb(value, strict_map_key=False)
         except:
             try:
                 self.cache.delete(key)
@@ -684,7 +684,7 @@ class StarryServer(object):
                 return (ret, data)
 
             try:
-                value = msgpack.loads(data, raw=False)
+                value = msgpack.unpackb(data, strict_map_key=False)
                 self._setCache(key, data)
             except:
                 (ret, value) = self._recover(key)
@@ -729,7 +729,7 @@ class StarryServer(object):
                 return (ret, value)
 
             try:
-                value = msgpack.loads(value, raw=False)
+                value = msgpack.unpackb(value, strict_map_key=False)
             except Exception as e:
                 self._recover(key)
                 return (False, f"Caught exception: {e.__class__}({e})")
@@ -741,7 +741,7 @@ class StarryServer(object):
                 return (True, "")
 
         try:
-            value = msgpack.dumps(value, use_bin_type=True)
+            value = msgpack.packb(value)
         except Exception as e:
             return (False, f"Caught exception: {e.__class__}({e})")
 
@@ -776,7 +776,7 @@ class StarryServer(object):
                 return (ret, value)
 
             try:
-                value = msgpack.loads(value, raw=False)
+                value = msgpack.unpackb(value, strict_map_key=False)
             except Exception as e:
                 self._recover(key)
                 return (False, f"Caught exception: {e.__class__}({e})")
@@ -788,7 +788,7 @@ class StarryServer(object):
                 return (True, "")
 
         try:
-            value = msgpack.dumps(value, use_bin_type=True)
+            value = msgpack.packb(value)
         except Exception as e:
             return (False, f"Caught exception: {e.__class__}({e})")
 
@@ -852,7 +852,7 @@ class StarryServer(object):
             return (ret, value)
 
         try:
-            value = msgpack.loads(value, raw=False)
+            value = msgpack.unpackb(value, strict_map_key=False)
         except Exception as e:
             self._recover(key)
             return (False, f"Caught exception: {e.__class__}({e})")
@@ -862,7 +862,7 @@ class StarryServer(object):
             return (False, f"Delete {key} failed: {msg}")
 
         try:
-            value = msgpack.dumps(value, use_bin_type=True)
+            value = msgpack.packb(value)
         except Exception as e:
             return (False, f"Caught exception: {e.__class__}({e})")
 
@@ -892,7 +892,7 @@ class StarryServer(object):
                 return (ret, data)
 
             try:
-                msgpack.loads(data, raw=False)
+                msgpack.unpackb(data, strict_map_key=False)
             except Exception as e:
                 self._recover(key)
                 return (False, f"Caught exception: {e.__class__}({e})")
@@ -913,7 +913,7 @@ class StarryServer(object):
             return (ret, value)
 
         try:
-            data = msgpack.loads(value, raw=True)
+            data = msgpack.unpackb(value, strict_map_key=False)
         except Exception as e:
             _removeFile(backup)
             _removeFile(f"{backup}.tmp")
@@ -940,7 +940,7 @@ class StarryServer(object):
             return (ret, value)
 
         try:
-            msgpack.loads(value, raw=False)
+            msgpack.unpackb(value, strict_map_key=False)
         except Exception as e:
             return (False, f"Caught exception: {e.__class__}({e})")
 
@@ -965,12 +965,17 @@ class StarryServer(object):
 
         return self._backup(key, check_flag=False)
 
+
+
     @contextmanager
     def _lock(self, mark):
         def _predicate():
             if len(self.queue) > 0:
                 if mark["mode"] == "r":
                     if any([q["mode"] == "w" for q in self.queue]):
+                        if self.write_first is True:
+                            return False
+
                         if None in mark["members"]:
                             return False
 
